package at.hl.okb.wsclient;

import static at.hl.okb.wsclient.Utilities.createNameFilter;

import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;

import at.hl.wsclient.okb.Algorithm;
import at.hl.wsclient.okb.AlgorithmParameter;
import at.hl.wsclient.okb.ArrayOfAlgorithm;
import at.hl.wsclient.okb.ArrayOfAlgorithmParameter;
import at.hl.wsclient.okb.ArrayOfProblem;
import at.hl.wsclient.okb.IOKBService;
import at.hl.wsclient.okb.OKBService;
import at.hl.wsclient.okb.ObjectFactory;
import at.hl.wsclient.okb.Platform;
import at.hl.wsclient.okb.Problem;

import com.google.common.collect.Iterables;

public class OKBServiceClient {
    OKBService srv = null;
    IOKBService port = null;

    // hack for allowing self signed certificates
    static {
	// Create a trust manager that does not validate certificate chains
	TrustManager[] trustAllCerts = new TrustManager[] { new X509TrustManager() {
	    public java.security.cert.X509Certificate[] getAcceptedIssuers() {
		return new java.security.cert.X509Certificate[] {};
	    }

	    public void checkClientTrusted(
		    java.security.cert.X509Certificate[] certs, String authType) {
	    }

	    public void checkServerTrusted(
		    java.security.cert.X509Certificate[] certs, String authType) {
	    }
	} };

	SSLContext sc;
	try {
	    sc = SSLContext.getInstance("SSL");
	    sc.init(null, trustAllCerts, new java.security.SecureRandom());
	    HttpsURLConnection
		    .setDefaultSSLSocketFactory(sc.getSocketFactory());
	} catch (NoSuchAlgorithmException e) {
	    throw new RuntimeException(e);
	} catch (KeyManagementException e) {
	    throw new RuntimeException(e);
	}

    }

    public OKBServiceClient() {
	this.srv = new OKBService();
	this.port = srv.getWSHttpBindingIOKBService1();
    }

    public Long getPlatformId(String platformName) {
	Platform returnValue = Iterables.find(
		port.getPlatforms().getPlatform(),
		createNameFilter(platformName));
	if (returnValue == null)
	    createPlatform(platformName);

	returnValue = Iterables.find(port.getPlatforms().getPlatform(),
		createNameFilter(platformName));
	return returnValue.getId();

    }

    private void createPlatform(String name) {

	ObjectFactory factory = new ObjectFactory();
	Platform platform = factory.createPlatform();

	platform.setName(factory.createNamedOKBItemName(name));
	port.addPlatform(platform);
    }

    public Long getAlgorithmId(String algorithmName) {
	Long algorithmId = null;

	ArrayOfAlgorithm allAlgorithms = port.getAlgorithms();
	for (Algorithm algo : allAlgorithms.getAlgorithm()) {
	    if (algo.getName().getValue() == algorithmName) {
		algorithmId = algo.getId();
	    }
	}
	return algorithmId;
    }

    public Long getProblemId(String problemName) {
	Long problemId = null;

	ArrayOfProblem allProblems = port.getProblems();
	for (Problem problem : allProblems.getProblem()) {
	    if (problem.getName().getValue() == problemName) {
		problemId = problem.getId();
	    }
	}
	return problemId;
    }

    public void checkAlgoParamsExists(Long algorithmId,
	    Map<String, Object> ecjAlgoParams) {

	List<String> missingParams = new ArrayList<String>();

	ArrayOfAlgorithmParameter hlAlgoParams = port
		.getAlgorithmParameters(algorithmId);
	Map<String, Object> hlAlgoParamMap = new HashMap<String, Object>();

	for (AlgorithmParameter algoParam : hlAlgoParams
		.getAlgorithmParameter()) {
	    hlAlgoParamMap.put(algoParam.getName().getValue(), null); // value
								      // hangs
								      // on
								      // Experiments
	}

	String[] ecjparamNames = (String[]) ecjAlgoParams.keySet().toArray();

	for (int i = 0; i < ecjparamNames.length; i++) {
	    if (!hlAlgoParamMap.containsKey(ecjparamNames[i])) {
		missingParams.add(new String(ecjparamNames[i]));
	    }
	}

    }

}
